@@ -7,16 +7,19 @@
 #define GFX_UTILS_H
 
 #include "gfxTypes.h"
-#include "gfxPattern.h"
-#include "gfxImageSurface.h"
-#include "ImageContainer.h"
-#include "mozilla/gfx/2D.h"
+#include "GraphicsFilter.h"
 #include "imgIContainer.h"
 
 class gfxDrawable;
 class nsIntRegion;
 struct nsIntRect;
 
+namespace mozilla {
+namespace layers {
+class PlanarYCbCrData;
+}
+}
+
 class gfxUtils {
 public:
     /*
@@ -26,7 +29,7 @@ public:
      * If aDestSurface is given, it must have identical format, dimensions, and
      * stride as the source.
      *
-     * If the source is not ImageFormatARGB32, no operation is performed.  If
+     * If the source is not gfxImageFormatARGB32, no operation is performed.  If
      * aDestSurface is given, the data is copied over.
      */
     static void PremultiplyImageSurface(gfxImageSurface *aSourceSurface,
@@ -57,8 +60,8 @@ public:
                                  const gfxRect&   aSourceRect,
                                  const gfxRect&   aImageRect,
                                  const gfxRect&   aFill,
-                                 const gfxImageSurface::gfxImageFormat aFormat,
-                                 gfxPattern::GraphicsFilter aFilter,
+                                 const gfxImageFormat aFormat,
+                                 GraphicsFilter aFilter,
                                  uint32_t         aImageFlags = imgIContainer::FLAG_NONE);
 
     /**
@@ -94,7 +97,7 @@ public:
     /*
      * Convert image format to depth value
      */
-    static int ImageFormatToDepth(gfxASurface::gfxImageFormat aFormat);
+    static int ImageFormatToDepth(gfxImageFormat aFormat);
 
     /**
      * Return the transform matrix that maps aFrom to the rectangle defined by
@@ -122,15 +125,15 @@ public:
     /**
      * Helper function for ConvertYCbCrToRGB that finds the
      * RGB buffer size and format for given YCbCrImage.
-     * @param aSuggestedFormat will be set to ImageFormatRGB24
+     * @param aSuggestedFormat will be set to gfxImageFormatRGB24
      *   if the desired format is not supported.
      * @param aSuggestedSize will be set to the picture size from aData
      *   if either the suggested size was {0,0}
      *   or simultaneous scaling and conversion is not supported.
      */
     static void
-    GetYCbCrToRGBDestFormatAndSize(const mozilla::layers::PlanarYCbCrImage::Data& aData,
-                                   gfxASurface::gfxImageFormat& aSuggestedFormat,
+    GetYCbCrToRGBDestFormatAndSize(const mozilla::layers::PlanarYCbCrData& aData,
+                                   gfxImageFormat& aSuggestedFormat,
                                    gfxIntSize& aSuggestedSize);
 
     /**
@@ -139,8 +142,8 @@ public:
      *   been passed to GetYCbCrToRGBDestFormatAndSize
      */
     static void
-    ConvertYCbCrToRGB(const mozilla::layers::PlanarYCbCrImage::Data& aData,
-                      const gfxASurface::gfxImageFormat& aDestFormat,
+    ConvertYCbCrToRGB(const mozilla::layers::PlanarYCbCrData& aData,
+                      const gfxImageFormat& aDestFormat,
                       const gfxIntSize& aDestSize,
                       unsigned char* aDestBuffer,
                       int32_t aStride);
