@@ -36,7 +36,7 @@ CompileScriptForPrincipalsVersionOrigin(JSContext *cx, JS::HandleObject obj,
     return script;
 }
 
-JSScript *
+static JSScript *
 FreezeThaw(JSContext *cx, JS::HandleScript script)
 {
     // freeze
@@ -58,7 +58,7 @@ GetScript(JSContext *cx, JS::HandleObject funobj)
     return JS_GetFunctionScript(cx, JS_GetObjectFunction(funobj));
 }
 
-JSObject *
+static JSObject *
 FreezeThaw(JSContext *cx, JS::HandleObject funobj)
 {
     // freeze
@@ -76,10 +76,8 @@ FreezeThaw(JSContext *cx, JS::HandleObject funobj)
     return funobj2;
 }
 
-static JSPrincipals testPrincipals[] = {
-    { 1 },
-    { 1 },
-};
+static TestJSPrincipals testPrincipal0(1);
+static TestJSPrincipals testPrincipal1(1);
 
 BEGIN_TEST(testXDR_principals)
 {
@@ -88,21 +86,21 @@ BEGIN_TEST(testXDR_principals)
     for (int i = TEST_FIRST; i != TEST_END; ++i) {
         // Appease the new JSAPI assertions. The stuff being tested here is
         // going away anyway.
-        JS_SetCompartmentPrincipals(compartment, &testPrincipals[0]);
-        script = createScriptViaXDR(&testPrincipals[0], nullptr, i);
+        JS_SetCompartmentPrincipals(compartment, &testPrincipal0);
+        script = createScriptViaXDR(&testPrincipal0, nullptr, i);
         CHECK(script);
-        CHECK(JS_GetScriptPrincipals(script) == &testPrincipals[0]);
-        CHECK(JS_GetScriptOriginPrincipals(script) == &testPrincipals[0]);
+        CHECK(JS_GetScriptPrincipals(script) == &testPrincipal0);
+        CHECK(JS_GetScriptOriginPrincipals(script) == &testPrincipal0);
 
-        script = createScriptViaXDR(&testPrincipals[0], &testPrincipals[0], i);
+        script = createScriptViaXDR(&testPrincipal0, &testPrincipal0, i);
         CHECK(script);
-        CHECK(JS_GetScriptPrincipals(script) == &testPrincipals[0]);
-        CHECK(JS_GetScriptOriginPrincipals(script) == &testPrincipals[0]);
+        CHECK(JS_GetScriptPrincipals(script) == &testPrincipal0);
+        CHECK(JS_GetScriptOriginPrincipals(script) == &testPrincipal0);
 
-        script = createScriptViaXDR(&testPrincipals[0], &testPrincipals[1], i);
+        script = createScriptViaXDR(&testPrincipal0, &testPrincipal1, i);
         CHECK(script);
-        CHECK(JS_GetScriptPrincipals(script) == &testPrincipals[0]);
-        CHECK(JS_GetScriptOriginPrincipals(script) == &testPrincipals[1]);
+        CHECK(JS_GetScriptPrincipals(script) == &testPrincipal0);
+        CHECK(JS_GetScriptOriginPrincipals(script) == &testPrincipal1);
     }
 
     return true;
@@ -163,7 +161,10 @@ BEGIN_TEST(testXDR_bug506491)
         "var f = makeClosure('0;', 'status', 'ok');\n";
 
     // compile
-    JS::RootedScript script(cx, JS_CompileScript(cx, global, s, strlen(s), __FILE__, __LINE__));
+    JS::CompileOptions options(cx);
+    options.setFileAndLine(__FILE__, __LINE__);
+    JS::RootedScript script(cx, JS_CompileScript(cx, global, s, strlen(s),
+                                                 options));
     CHECK(script);
 
     script = FreezeThaw(cx, script);
@@ -187,7 +188,9 @@ END_TEST(testXDR_bug506491)
 BEGIN_TEST(testXDR_bug516827)
 {
     // compile an empty script
-    JS::RootedScript script(cx, JS_CompileScript(cx, global, "", 0, __FILE__, __LINE__));
+    JS::CompileOptions options(cx);
+    options.setFileAndLine(__FILE__, __LINE__);
+    JS::RootedScript script(cx, JS_CompileScript(cx, global, "", 0, options));
     CHECK(script);
 
     script = FreezeThaw(cx, script);
@@ -208,7 +211,10 @@ BEGIN_TEST(testXDR_source)
         nullptr
     };
     for (const char **s = samples; *s; s++) {
-        JS::RootedScript script(cx, JS_CompileScript(cx, global, *s, strlen(*s), __FILE__, __LINE__));
+        JS::CompileOptions options(cx);
+        options.setFileAndLine(__FILE__, __LINE__);
+        JS::RootedScript script(cx, JS_CompileScript(cx, global, *s, strlen(*s),
+                                                     options));
         CHECK(script);
         script = FreezeThaw(cx, script);
         CHECK(script);
@@ -231,7 +237,9 @@ BEGIN_TEST(testXDR_sourceMap)
     };
     JS::RootedScript script(cx);
     for (const char **sm = sourceMaps; *sm; sm++) {
-        script = JS_CompileScript(cx, global, "", 0, __FILE__, __LINE__);
+        JS::CompileOptions options(cx);
+        options.setFileAndLine(__FILE__, __LINE__);
+        script = JS_CompileScript(cx, global, "", 0, options);
         CHECK(script);
 
         size_t len = strlen(*sm);
