@@ -4,27 +4,24 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#ifndef String_inl_h__
-#define String_inl_h__
+#ifndef vm_String_inl_h
+#define vm_String_inl_h
+
+#include "vm/String.h"
 
 #include "mozilla/PodOperations.h"
 
 #include "jscntxt.h"
-#include "jsprobes.h"
 
 #include "gc/Marking.h"
-#include "String.h"
 
 #include "jsgcinlines.h"
-#include "jsobjinlines.h"
-#include "gc/Barrier-inl.h"
-#include "gc/StoreBuffer.h"
 
 namespace js {
 
 template <AllowGC allowGC>
 static JS_ALWAYS_INLINE JSInlineString *
-NewShortString(JSContext *cx, JS::Latin1Chars chars)
+NewShortString(ThreadSafeContext *cx, JS::Latin1Chars chars)
 {
     size_t len = chars.length();
     JS_ASSERT(JSShortString::lengthFits(len));
@@ -32,7 +29,7 @@ NewShortString(JSContext *cx, JS::Latin1Chars chars)
                           ? JSInlineString::new_<allowGC>(cx)
                           : JSShortString::new_<allowGC>(cx);
     if (!str)
-        return NULL;
+        return nullptr;
 
     jschar *p = str->init(len);
     for (size_t i = 0; i < len; ++i)
@@ -43,7 +40,7 @@ NewShortString(JSContext *cx, JS::Latin1Chars chars)
 
 template <AllowGC allowGC>
 static JS_ALWAYS_INLINE JSInlineString *
-NewShortString(JSContext *cx, JS::StableTwoByteChars chars)
+NewShortString(ExclusiveContext *cx, JS::StableTwoByteChars chars)
 {
     size_t len = chars.length();
 
@@ -56,7 +53,7 @@ NewShortString(JSContext *cx, JS::StableTwoByteChars chars)
                           ? JSInlineString::new_<allowGC>(cx)
                           : JSShortString::new_<allowGC>(cx);
     if (!str)
-        return NULL;
+        return nullptr;
 
     jschar *storage = str->init(len);
     mozilla::PodCopy(storage, chars.start().get(), len);
@@ -66,7 +63,7 @@ NewShortString(JSContext *cx, JS::StableTwoByteChars chars)
 
 template <AllowGC allowGC>
 static JS_ALWAYS_INLINE JSInlineString *
-NewShortString(JSContext *cx, JS::TwoByteChars chars)
+NewShortString(ExclusiveContext *cx, JS::TwoByteChars chars)
 {
     size_t len = chars.length();
 
@@ -80,7 +77,7 @@ NewShortString(JSContext *cx, JS::TwoByteChars chars)
                           : JSShortString::new_<NoGC>(cx);
     if (!str) {
         if (!allowGC)
-            return NULL;
+            return nullptr;
         jschar tmp[JSShortString::MAX_SHORT_LENGTH];
         mozilla::PodCopy(tmp, chars.start().get(), len);
         return NewShortString<CanGC>(cx, JS::StableTwoByteChars(tmp, len));
@@ -93,63 +90,19 @@ NewShortString(JSContext *cx, JS::TwoByteChars chars)
 }
 
 static inline void
-StringWriteBarrierPost(JSRuntime *rt, JSString **strp)
+StringWriteBarrierPost(js::ThreadSafeContext *maybecx, JSString **strp)
 {
 }
 
 static inline void
-StringWriteBarrierPostRemove(JSRuntime *rt, JSString **strp)
+StringWriteBarrierPostRemove(js::ThreadSafeContext *maybecx, JSString **strp)
 {
 }
 
 } /* namespace js */
 
-inline void
-JSString::writeBarrierPre(JSString *str)
-{
-#ifdef JSGC_INCREMENTAL
-    if (!str || !str->runtime()->needsBarrier())
-        return;
-
-    JS::Zone *zone = str->zone();
-    if (zone->needsBarrier()) {
-        JSString *tmp = str;
-        MarkStringUnbarriered(zone->barrierTracer(), &tmp, "write barrier");
-        JS_ASSERT(tmp == str);
-    }
-#endif
-}
-
-inline void
-JSString::writeBarrierPost(JSString *str, void *addr)
-{
-}
-
-inline bool
-JSString::needWriteBarrierPre(JS::Zone *zone)
-{
-#ifdef JSGC_INCREMENTAL
-    return zone->needsBarrier();
-#else
-    return false;
-#endif
-}
-
-inline void
-JSString::readBarrier(JSString *str)
-{
-#ifdef JSGC_INCREMENTAL
-    JS::Zone *zone = str->zone();
-    if (zone->needsBarrier()) {
-        JSString *tmp = str;
-        MarkStringUnbarriered(zone->barrierTracer(), &tmp, "read barrier");
-        JS_ASSERT(tmp == str);
-    }
-#endif
-}
-
 JS_ALWAYS_INLINE bool
-JSString::validateLength(JSContext *maybecx, size_t length)
+JSString::validateLength(js::ThreadSafeContext *maybecx, size_t length)
 {
     if (JS_UNLIKELY(length > JSString::MAX_LENGTH)) {
         js_ReportAllocationOverflow(maybecx);
@@ -160,28 +113,28 @@ JSString::validateLength(JSContext *maybecx, size_t length)
 }
 
 JS_ALWAYS_INLINE void
-JSRope::init(JSString *left, JSString *right, size_t length)
+JSRope::init(js::ThreadSafeContext *cx, JSString *left, JSString *right, size_t length)
 {
     d.lengthAndFlags = buildLengthAndFlags(length, ROPE_FLAGS);
     d.u1.left = left;
     d.s.u2.right = right;
-    js::StringWriteBarrierPost(runtime(), &d.u1.left);
-    js::StringWriteBarrierPost(runtime(), &d.s.u2.right);
+    js::StringWriteBarrierPost(cx, &d.u1.left);
+    js::StringWriteBarrierPost(cx, &d.s.u2.right);
 }
 
 template <js::AllowGC allowGC>
 JS_ALWAYS_INLINE JSRope *
-JSRope::new_(JSContext *cx,
+JSRope::new_(js::ThreadSafeContext *cx,
              typename js::MaybeRooted<JSString*, allowGC>::HandleType left,
              typename js::MaybeRooted<JSString*, allowGC>::HandleType right,
              size_t length)
 {
     if (!validateLength(cx, length))
-        return NULL;
+        return nullptr;
     JSRope *str = (JSRope *) js_NewGCString<allowGC>(cx);
     if (!str)
-        return NULL;
-    str->init(left, right, length);
+        return nullptr;
+    str->init(cx, left, right, length);
     return str;
 }
 
@@ -193,17 +146,19 @@ JSRope::markChildren(JSTracer *trc)
 }
 
 JS_ALWAYS_INLINE void
-JSDependentString::init(JSLinearString *base, const jschar *chars, size_t length)
+JSDependentString::init(js::ThreadSafeContext *cx, JSLinearString *base, const jschar *chars,
+                        size_t length)
 {
     JS_ASSERT(!js::IsPoisonedPtr(base));
     d.lengthAndFlags = buildLengthAndFlags(length, DEPENDENT_FLAGS);
     d.u1.chars = chars;
     d.s.u2.base = base;
-    js::StringWriteBarrierPost(runtime(), reinterpret_cast<JSString **>(&d.s.u2.base));
+    js::StringWriteBarrierPost(cx, reinterpret_cast<JSString **>(&d.s.u2.base));
 }
 
 JS_ALWAYS_INLINE JSLinearString *
-JSDependentString::new_(JSContext *cx, JSLinearString *baseArg, const jschar *chars, size_t length)
+JSDependentString::new_(js::ExclusiveContext *cx,
+                        JSLinearString *baseArg, const jschar *chars, size_t length)
 {
     /* Try to avoid long chains of dependent strings. */
     while (baseArg->isDependent())
@@ -235,7 +190,7 @@ JSDependentString::new_(JSContext *cx, JSLinearString *baseArg, const jschar *ch
 
     JSDependentString *str = (JSDependentString *)js_NewGCString<js::NoGC>(cx);
     if (str) {
-        str->init(baseArg, chars, length);
+        str->init(cx, baseArg, chars, length);
         return str;
     }
 
@@ -243,8 +198,8 @@ JSDependentString::new_(JSContext *cx, JSLinearString *baseArg, const jschar *ch
 
     str = (JSDependentString *)js_NewGCString<js::CanGC>(cx);
     if (!str)
-        return NULL;
-    str->init(base, chars, length);
+        return nullptr;
+    str->init(cx, base, chars, length);
     return str;
 }
 
@@ -264,19 +219,12 @@ JSFlatString::toPropertyName(JSContext *cx)
 #endif
     if (isAtom())
         return asAtom().asPropertyName();
-    JSAtom *atom = js::AtomizeString<js::CanGC>(cx, this);
+    JSAtom *atom = js::AtomizeString(cx, this);
     if (!atom)
-        return NULL;
+        return nullptr;
     return atom->asPropertyName();
 }
 
-JS_ALWAYS_INLINE JSAtom *
-JSFlatString::morphAtomizedStringIntoAtom()
-{
-    d.lengthAndFlags = buildLengthAndFlags(length(), ATOM_BIT);
-    return &asAtom();
-}
-
 JS_ALWAYS_INLINE void
 JSStableString::init(const jschar *chars, size_t length)
 {
@@ -286,22 +234,22 @@ JSStableString::init(const jschar *chars, size_t length)
 
 template <js::AllowGC allowGC>
 JS_ALWAYS_INLINE JSStableString *
-JSStableString::new_(JSContext *cx, const jschar *chars, size_t length)
+JSStableString::new_(js::ThreadSafeContext *cx, const jschar *chars, size_t length)
 {
     JS_ASSERT(chars[length] == jschar(0));
 
     if (!validateLength(cx, length))
-        return NULL;
+        return nullptr;
     JSStableString *str = (JSStableString *)js_NewGCString<allowGC>(cx);
     if (!str)
-        return NULL;
+        return nullptr;
     str->init(chars, length);
     return str;
 }
 
 template <js::AllowGC allowGC>
 JS_ALWAYS_INLINE JSInlineString *
-JSInlineString::new_(JSContext *cx)
+JSInlineString::new_(js::ThreadSafeContext *cx)
 {
     return (JSInlineString *)js_NewGCString<allowGC>(cx);
 }
@@ -324,7 +272,7 @@ JSInlineString::resetLength(size_t length)
 
 template <js::AllowGC allowGC>
 JS_ALWAYS_INLINE JSShortString *
-JSShortString::new_(JSContext *cx)
+JSShortString::new_(js::ThreadSafeContext *cx)
 {
     return js_NewGCShortString<allowGC>(cx);
 }
@@ -346,68 +294,23 @@ JSExternalString::new_(JSContext *cx, const jschar *chars, size_t length,
     JS_ASSERT(chars[length] == 0);
 
     if (!validateLength(cx, length))
-        return NULL;
+        return nullptr;
     JSExternalString *str = js_NewGCExternalString(cx);
     if (!str)
-        return NULL;
+        return nullptr;
     str->init(chars, length, fin);
-    cx->runtime->updateMallocCounter(cx->compartment->zone(), (length + 1) * sizeof(jschar));
+    cx->runtime()->updateMallocCounter(cx->zone(), (length + 1) * sizeof(jschar));
     return str;
 }
 
-inline bool
-js::StaticStrings::fitsInSmallChar(jschar c)
-{
-    return c < SMALL_CHAR_LIMIT && toSmallChar[c] != INVALID_SMALL_CHAR;
-}
-
-inline bool
-js::StaticStrings::hasUnit(jschar c)
-{
-    return c < UNIT_STATIC_LIMIT;
-}
-
-inline JSAtom *
-js::StaticStrings::getUnit(jschar c)
-{
-    JS_ASSERT(hasUnit(c));
-    return unitStaticTable[c];
-}
-
-inline bool
-js::StaticStrings::hasUint(uint32_t u)
-{
-    return u < INT_STATIC_LIMIT;
-}
-
-inline JSAtom *
-js::StaticStrings::getUint(uint32_t u)
-{
-    JS_ASSERT(hasUint(u));
-    return intStaticTable[u];
-}
-
-inline bool
-js::StaticStrings::hasInt(int32_t i)
-{
-    return uint32_t(i) < INT_STATIC_LIMIT;
-}
-
-inline JSAtom *
-js::StaticStrings::getInt(int32_t i)
-{
-    JS_ASSERT(hasInt(i));
-    return getUint(uint32_t(i));
-}
-
 inline JSLinearString *
 js::StaticStrings::getUnitStringForElement(JSContext *cx, JSString *str, size_t index)
 {
     JS_ASSERT(index < str->length());
-    const jschar *chars = str->getChars(cx);
-    if (!chars)
-        return NULL;
-    jschar c = chars[index];
+
+    jschar c;
+    if (!str->getChar(cx, index, &c))
+        return nullptr;
     if (c < UNIT_STATIC_LIMIT)
         return getUnit(c);
     return js_NewDependentString(cx, str, index, 1);
@@ -422,50 +325,6 @@ js::StaticStrings::getLength2(jschar c1, jschar c2)
     return length2StaticTable[index];
 }
 
-inline JSAtom *
-js::StaticStrings::getLength2(uint32_t i)
-{
-    JS_ASSERT(i < 100);
-    return getLength2('0' + i / 10, '0' + i % 10);
-}
-
-/* Get a static atomized string for chars if possible. */
-inline JSAtom *
-js::StaticStrings::lookup(const jschar *chars, size_t length)
-{
-    switch (length) {
-      case 1:
-        if (chars[0] < UNIT_STATIC_LIMIT)
-            return getUnit(chars[0]);
-        return NULL;
-      case 2:
-        if (fitsInSmallChar(chars[0]) && fitsInSmallChar(chars[1]))
-            return getLength2(chars[0], chars[1]);
-        return NULL;
-      case 3:
-        /*
-         * Here we know that JSString::intStringTable covers only 256 (or at least
-         * not 1000 or more) chars. We rely on order here to resolve the unit vs.
-         * int string/length-2 string atom identity issue by giving priority to unit
-         * strings for "0" through "9" and length-2 strings for "10" through "99".
-         */
-        JS_STATIC_ASSERT(INT_STATIC_LIMIT <= 999);
-        if ('1' <= chars[0] && chars[0] <= '9' &&
-            '0' <= chars[1] && chars[1] <= '9' &&
-            '0' <= chars[2] && chars[2] <= '9') {
-            int i = (chars[0] - '0') * 100 +
-                      (chars[1] - '0') * 10 +
-                      (chars[2] - '0');
-
-            if (unsigned(i) < INT_STATIC_LIMIT)
-                return getInt(i);
-        }
-        return NULL;
-    }
-
-    return NULL;
-}
-
 JS_ALWAYS_INLINE void
 JSString::finalize(js::FreeOp *fop)
 {
@@ -513,4 +372,4 @@ JSExternalString::finalize(js::FreeOp *fop)
     fin->finalize(fin, const_cast<jschar *>(chars()));
 }
 
-#endif
+#endif /* vm_String_inl_h */
